Package: declared
Title: Functions for Declared Missing Values
Version: 0.26
Authors@R: 
    c(
        person(
            "Adrian", "Dusa",
            email = "dusa.adrian@unibuc.ro",
            role = c("aut", "cre", "cph"),
            comment = c(ORCID = "0000-0002-3525-9253")
        )
    )
URL: https://github.com/dusadrian/declared
BugReports: https://github.com/dusadrian/declared/issues
Description: A zero dependency package containing functions to declare labels
    and missing values, coupled with associated functions to create (weighted)
    tables of frequencies and various other summary measures. 
    Some of the base functions have been rewritten to make use of the specific 
    information about the missing values, most importantly to distinguish
    between empty NA and declared NA values.
    Some functions have similar functionality with the corresponding ones from
    packages "haven" and "labelled". The aim is to ensure as much compatibility
    as possible with these packages, while offering an alternative in the
    objects of class "declared".
License: GPL (>= 3)
Language: en-US
Encoding: UTF-8
NeedsCompilation: yes
Packaged: 2026-04-01 16:01:14 UTC; dusadrian
Author: Adrian Dusa [aut, cre, cph] (ORCID:
    <https://orcid.org/0000-0002-3525-9253>)
Maintainer: Adrian Dusa <dusa.adrian@unibuc.ro>
Repository: CRAN
Date/Publication: 2026-04-02 10:30:02 UTC
Built: R 4.6.0; x86_64-apple-darwin20; 2026-04-26 13:32:21 UTC; unix
Archs: declared.so.dSYM
