# Copyright (c) 2022 - 2026, Adrian Dusa
# All rights reserved.
# 
# Redistribution and use in source and binary forms, with or without
# modification, in whole or in part, are permitted provided that the
# following conditions are met:
#     * Redistributions of source code must retain the above copyright
#       notice, this list of conditions and the following disclaimer.
#     * Redistributions in binary form must reproduce the above copyright
#       notice, this list of conditions and the following disclaimer in the
#       documentation and/or other materials provided with the distribution.
#     * The names of its contributors may NOT be used to endorse or promote
#       products derived from this software without specific prior written
#       permission.
# 
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
# AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
# IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
# ARE DISCLAIMED. IN NO EVENT SHALL ADRIAN DUSA BE LIABLE FOR ANY
# DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
# (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
# LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
# ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
# (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
# SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

#' @rdname weighted
#' @order 11
#' @export
`wsummary` <- function (x, wt = NULL, ...) {

    wsum <- wquantile (x, wt = wt, ... = ...)
    wmean <- wmean (x, wt = wt, ... = ...)
    wsum <- c (wsum[1:3], wmean, wsum[4:5])

    if (inherits (x, "haven_labelled")) {
        x <- as.declared (x)
    }

    nas <- wtable (is.empty (x), wt = wt)
    rnms <- rownames (nas)
    if (is.null (rnms)) {
        rnms <- names (nas)
    }

    names (wsum) <- c ("Min.", "1st Qu.", "Median", "Mean", "3rd Qu.", "Max.")

    if (any (rnms == "TRUE")) {
        wsum <- c (
            wsum,
            "NA's" = attr (nas, "toprint")$fre[which (rnms == "TRUE")[1]]
        )
    }

    class (wsum) <- c ("fobject", class (wsum))
    return (wsum)
}

#' @rdname declared_internal
#' @keywords internal
#' @export
`w_summary` <- function (...) {
    wsummary(...)
}
